const Balance = require('../../constants/Balance');
const BattleCalc = require('../BattleCalc');
const DeathEvent = require('./DeathEvent');
const { logError } = require('../logger');

const DamageEvent = function(battleState, actor, dmg, cause) {
  try {
    this.eventHandle = 'damage';
    this.cause = cause;

    this.damage = dmg - 2 + Math.random() * 5;

    // check for crit
    let crit_chance = 0.1;
    // from weapon
    if (actor && actor.equipment) {
      if (actor.equipment.weapon?.suffix === 'destruction') {
        crit_chance += 0.2;
      }
    }
    // TODO - crit chance from stats?
    if (Math.random() <= crit_chance) {
      this.crit = true;
      this.damage *= Balance.CRITICAL_STRIKE_MOD;
    }

    // apply equipment damage mods
    let equipment_damage_mod = 1.0;
    for (const item of Object.values(actor?.equipment ?? {})) {
      if (cause.physical) {
        if (item?.physDamageMod) {
          equipment_damage_mod += item.physDamageMod;
        }
      }

      if (cause.magic) {
        if (item?.magicDamageMod) {
          equipment_damage_mod += item.magicDamageMod;
        }
        if (item?.type==='staff' && item?.damageMod) {
          equipment_damage_mod += item.damageMod;
        }
      }

      if (cause.melee) {
        if (item?.meleeDamageMod) {
          equipment_damage_mod += item.meleeDamageMod;
        }
        if (item?.slot==='weapon' && item?.damageMod) {
          equipment_damage_mod += item.damageMod;
        }
      }

      if (cause.bow) {
        if (item?.bowDamageMod) {
          equipment_damage_mod += item.bowDamageMod;
        }
        if (item?.type==='bow' && item?.damageMod) {
          equipment_damage_mod += item.damageMod;
        }
      }
    }
    this.damage *= equipment_damage_mod;

    this.applyStanceAdjustments = (attacker) => {
      try {
        this.damage = applyAttackerStanceEffects(attacker, this.damage);
        this.damage = applyDefenderStanceEffects(actor, this.damage);
      } catch (err) {
        logError(err, {
          module: 'DamageEvent',
          func: 'applyStanceAdjustments',
          actor,
          attacker,
          dmg,
          cause,
        });
      }
      return this.damage;
    };
    const applyAttackerStanceEffects = (attacker, prev_damage) => {
      var ds = BattleCalc.getCondition(attacker, 'defensive_stance');
      var os = BattleCalc.getCondition(attacker, 'offensive_stance');
      if (ds)  {
        return prev_damage * (0.50 + 0.03 * ds.potency);
      } else if (os) {
        return prev_damage * (1.00 + 0.03 * os.potency);
      }
      return prev_damage;
    };
    const applyDefenderStanceEffects = (defender, prev_damage) => {
      var ds = BattleCalc.getCondition(defender, 'defensive_stance');
      var os = BattleCalc.getCondition(defender, 'offensive_stance');
      if (ds)  {
        return prev_damage * (1.00 - 0.03 * ds.potency);
      } else if (os) {
        return prev_damage * (1.50 - 0.03 * os.potency);
      }
      return prev_damage;
    };

    this.applyArmorAdjustment = (actor, victim) => {
      try {
        if (!victim || !victim.attributes) {
          return this.damage;
        }

        let armorTotal = victim.attributes.armorClass;
        const { equipment: { weapon, off_hand } } = actor;

        if (cause.handle === 'lightning') {
          armorTotal = 0;
        } else if (
          (cause.weapon && weapon?.suffix === 'penetration')
          || (cause.bow && off_hand?.type === 'bow' && off_hand.suffix === 'penetration')
        ) {
          armorTotal *= 0.5;
        }

        this.damage -= armorTotal;

        return this.damage;
      } catch (err) {
        logError(err, {
          module: 'DamageEvent',
          func: 'applyArmorAdjustment',
          actor,
          victim,
          dmg,
          cause,
        });
      }
    };

    this.execute = (victim) => {
      try {
        if (!actor || !victim) {
          return;
        }

        this.victimId = victim.uid;

        // finalize as an integer
        this.damage = actor.handle === 'poppet' ? 0 : Math.max(0, Math.floor(this.damage));
        if (isNaN(this.damage)) {
          return;
        }

        victim.hp -= Math.min(this.damage, victim.hp);

        this.affectedPieces = [victim];
        battleState.onBattleEvent(this);

        if (isNaN(victim.hp) || victim.hp <= 0) {
          const death = new DeathEvent(battleState);
          death.execute(victim);
        }
      } catch (err) {
        logError(err, {
          module: 'DamageEvent',
          func: 'execute',
          actor,
          victim,
          dmg,
          cause,
        });
      }
    };
  } catch (err) {
    logError(err, {
      module: 'DamageEvent',
      func: 'constructor',
      battleState,
      actor,
      dmg,
      cause,
    });
  }
};
module.exports = DamageEvent;
